"{ Package: 'jn:refactoring_custom' }"

CustomCodeGenerator subclass:#CustomAccessMethodsCodeGenerator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!


!CustomAccessMethodsCodeGenerator class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ 'Generates three bogus methods - foo, bar and baz'

    "Modified: / 26-01-2014 / 21:46:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ 'foo, bar, baz...'

    "Modified: / 26-01-2014 / 21:45:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomAccessMethodsCodeGenerator class methodsFor:'queries'!

availableInContext: aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise. 

     Called by the UI to figure out what generators / refactorings 
     are available at given point. See class CustomContext for details."

    ^ aCustomContext selectedClasses notEmptyOrNil

    "Created: / 26-01-2014 / 21:41:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-01-2014 / 23:39:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-03-2014 / 22:34:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

availableInPerspective:aCustomPerspective 
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.

     Called by the UI to figure out what generators / refactorings
     to show"

    ^ aCustomPerspective isClassPerspective

    "Created: / 26-01-2014 / 21:40:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 04-03-2014 / 21:41:34 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomAccessMethodsCodeGenerator methodsFor:'code generation-individual methods'!

createAccessMethodsFor:aCollectionOfVarNames in:aClass withChange:withChange asValueHolder:asValueHolder readersOnly:readersOnly writersOnly:writersOnly lazyInitialization:lazyInitialization
    "workhorse for creating access methods for instvars."

    |classesClassVars generateCommentsForSetters generateCommentsForGetters|

    self startCollectChanges.

    generateCommentsForSetters := userPreferences generateCommentsForSetters.
    generateCommentsForGetters := userPreferences generateCommentsForGetters.

    classesClassVars := aClass theNonMetaclass allClassVarNames.

    aCollectionOfVarNames do:[:name |
        |source varType methodName defaultMethodName argName|

        varType := (classesClassVars includes:name) 
                        ifTrue:['static'] 
                        ifFalse:[
                            (aClass isMeta ifTrue:['classInstVar'] ifFalse:['instance'])].

        methodName := name.
        name isUppercaseFirst ifTrue:[
            (name conform:[:ch | ch isLetter not or:[ch isUppercase]]) ifFalse:[      "/ allow all-uppercase for class-vars
                methodName := methodName asLowercaseFirst. 
            ]
        ].
        argName := 'something'.

        "/ the GETTER
        writersOnly ifFalse:[
            lazyInitialization ifTrue:[
                defaultMethodName := 'default' , name asUppercaseFirst.
            ].

            "check, if method is not already present"
            (aClass includesSelector:(methodName asSymbol)) ifFalse:[
                asValueHolder ifTrue:[
                    source := methodName , '\'.
                    generateComments ifTrue:[
                        source := source , '    "return/create the ''%2'' value holder (automatically generated)"\\'. 
                    ].
                    source := source , '    %2 isNil ifTrue:[\'.
                    lazyInitialization ifTrue:[
                        source := source
                                   , '        %2 := self class %3 asValue.\'.
                    ] ifFalse:[
                        source := source
                                   , '        %2 := ValueHolder new.\'.
                    ].

                    withChange ifTrue:[
                    source := source
                               , '        %2 addDependent:self.\'.
                    ].
                    source := source
                               , '    ].\'
                               , '    ^ %2'.
                ] ifFalse:[
                    source := methodName , '\'.
                    lazyInitialization ifTrue:[
                        generateCommentsForGetters ifTrue:[
                            source := source , '    "return the %1 instance variable ''%2'' with lazy instance creation (automatically generated)"\\'. 
                        ].
                        source := source
                                    , '    %2 isNil ifTrue:[\'
                                    , '        %2 := self class %3.\'
                                    , '    ].\'
                                    , '    ^ %2'.
                    ] ifFalse:[
                        generateCommentsForGetters ifTrue:[
                            source := source , '    "return the %1 instance variable ''%2'' (automatically generated)"\\'. 
                        ].
                        source := source
                                    , '    ^ %2'.
                    ].
                ].
                source := (source bindWith:varType with:name with:defaultMethodName) withCRs.
                self compile:source forClass:aClass inCategory:(asValueHolder ifTrue:[#aspects] ifFalse:[#accessing]).
            ] ifTrue:[
                Transcript showCR:'method ''', methodName , ''' already present'
            ].

            "/ default for lazy on class side
            lazyInitialization ifTrue:[
                (aClass theMetaclass includesSelector:(defaultMethodName asSymbol)) ifFalse:[
                    source := defaultMethodName , '\'.
                    generateComments ifTrue:[
                        source := source , '    "default value for the ''%2'' instance variable (automatically generated)"\\'. 
                    ].
                    source := source    
                               , '    self shouldImplement.\'
                               , '    ^ nil.'.
                    source := (source bindWith:varType with:name) withCRs.
                    self compile:source forClass:aClass theMetaclass inCategory:#defaults.
                ].
            ].
        ].

        "/ the SETTER
        readersOnly ifFalse:[
            (aClass includesSelector:((methodName , ':') asSymbol)) ifFalse:[
                ((methodName size > 2) and:[ (methodName startsWith:'is') and:[ (methodName at:3) isUppercase ]])
                ifTrue:[
                    argName := 'aBoolean'
                ].
                asValueHolder ifTrue:[
                    source := methodName , ':%3\'.  "/ argName
                    generateComments ifTrue:[
                        source := source , '    "set the ''%2'' value holder' , ' (automatically generated)"\\'.
                    ].
                    withChange ifTrue:[
                        source := source
                                  , '    |oldValue newValue|\\'
                                  , '    %2 notNil ifTrue:[\'
                                  , '        oldValue := %2 value.\'
                                  , '        %2 removeDependent:self.\'
                                  , '    ].\'
                                  , '    %2 := %3.\'        "/ argName
                                  , '    %2 notNil ifTrue:[\'
                                  , '        %2 addDependent:self.\'
                                  , '    ].\'
                                  , '    newValue := %2 value.\'
                                  , '    oldValue ~~ newValue ifTrue:[\'
                                  , '        self update:#value with:newValue from:%2.\'
                                  , '    ].\'
                    ] ifFalse:[
                        source := source 
                                  , '    %2 := %3.'.  "/ argName
                    ].
                ] ifFalse:[
                    source := methodName , ':%3\'.    "/ argName
                    withChange ifTrue:[
                        generateComments ifTrue:[
                            source := source , '    "set the value of the %1 variable ''%2'''.
                            source := source , ' and send a change notification (automatically generated)"\\'.
                        ].
                        source := source
                                  , '    (%2 ~~ %3) ifTrue:[\'
                                  , '        %2 := %3.\'           "/ argName
                                  , '        self changed:#%2.\'
                                  , '     ].\'.
                    ] ifFalse:[
                        generateCommentsForSetters ifTrue:[
                            source := source , '    "set the value of the %1 variable ''%2'''.
                            source := source , ' (automatically generated)"\\'.
                        ].
                        source := source
                                  , '    %2 := %3.'.          "/ argName
                    ].
                ].
                source := (source bindWith:varType with:name with:argName) withCRs.
                self 
                    compile:source 
                    forClass:aClass 
                    inCategory:(asValueHolder ifTrue:[#aspects] ifFalse:[#accessing]).
            ] ifTrue:[
                Transcript showCR:'method ''', methodName , ':'' already present'
            ].
        ].
    ].

    self executeCollectedChangesNamed:('Add Accessors').

    "Created: / 12-03-2014 / 23:35:18 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createSetterFor:aVarName in:aClass 
    self 
        createAccessMethodsFor:{ aVarName} in:aClass 
        withChange:false asValueHolder:false readersOnly:false writersOnly:true 
        lazyInitialization:false

    "Created: / 14-03-2014 / 23:12:46 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

setterMethodFor:aVarName 
    |source methodName argName comment|

    comment := ''.
    userPreferences generateCommentsForSetters ifTrue:[
        comment := '    "set the value of the %1 variable ''%2'' (automatically generated)"\\'.
    ].

    methodName := aVarName.
    argName := 'something'.
    source := methodName , ':%3\'.  "/ argName 
    source := source , '    %2 := %3.'. 

    ^ source

    "Created: / 15-03-2014 / 22:13:00 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomAccessMethodsCodeGenerator methodsFor:'executing'!

executeInContext:aCustomContext
    "Generates the code or perform the refactoring. This method
     is guaranteed to be called only if #availableInContext: returns
     true.
     
     Called by the UI when user want to perform the task"

    | classes |

    classes := aCustomContext selectedClasses.
    classes do:[:class | 
        (class canUnderstand: #foo) ifFalse:[  
            self compile: 'foo
    ^ self bar' forClass: class inCategory: 'foo - bar - baz'
        ].
        (class canUnderstand: #bar) ifFalse:[  
            self compile: 'bar
    ^ self baz' forClass: class inCategory: 'foo - bar - baz'
        ].
        (class canUnderstand: #baz) ifFalse:[  
            self compile: 'baz
    ^ 42' forClass: class inCategory: 'foo - bar - baz'
        ].

    ].

    "Modified: / 26-01-2014 / 23:41:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CustomAccessMethodsCodeGenerator class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

