"{ Package: 'jn:refactoring_custom' }"

TestCase subclass:#CustomSourceCodeBuilderTests
	instanceVariableNames:'changeManager builder'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomSourceCodeBuilderTests methodsFor:'initialization & release'!

setUp

    changeManager := RefactoryChangeManager instance.
    builder := CustomSourceCodeBuilder new.

    "Created: / 16-04-2014 / 21:39:50 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

tearDown

    builder redoChanges

    "Created: / 16-04-2014 / 21:39:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 27-04-2014 / 17:09:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilderTests methodsFor:'private'!

lastUndoChange

    changeManager undoableOperations isEmpty ifTrue: [ 
        ^ nil 
    ] ifFalse: [
        ^ changeManager undoChange
    ].

    "Created: / 16-04-2014 / 21:40:37 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSourceCodeBuilderTests methodsFor:'tests'!

test_chaining_api_create_method
    | class |

    class := self class.

    builder
        class: class;
        protocol: 'a protocol';
        source: '`@selector
            self shouldImplement';
        replace: '`@selector' with: 'aSelector';
        createMethod;

        replace: '`@selector' with: 'aSelector2';
        createMethod.

    self assert: (class includesSelector: #aSelector) not.
    self assert: (class includesSelector: #aSelector2) not.

    builder execute.

    self assert: (class includesSelector: #aSelector).
    self assert: (class includesSelector: #aSelector2).

    self assert: (class sourceCodeAt: #aSelector) = 'aSelector
    self shouldImplement'.

    self assert: (class sourceCodeAt: #aSelector2) = 'aSelector2
    self shouldImplement'.

    "Created: / 27-04-2014 / 17:48:30 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 28-04-2014 / 09:19:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_chaining_api_create_method_with_class
    | class className method |

    className := 'TestClassNameForTestCase'.

    class := builder
        className: className;
        classVariableNames: 'ClassVar1 ClassVar2';
        instanceVariableNames: 'instVar1 instVar2';
        poolDictionaryNames: 'dictName';
        superclassName: 'TestCase';
        category: 'Some-Category';
        createClass.

    builder 
        class: class;
        protocol: 'a protocol';
        source: '`@selector
            self shouldImplement';
        replace: '`@selector' with: 'aSelector';
        createMethod.

    self assert: (Smalltalk classNamed: className) isNil.

    builder execute.

    class := Smalltalk classNamed: className.

    self assert: class new className = className.
    self assert: class superclass new className = 'TestCase'.
    self assert: class instanceVariableString = 'instVar1 instVar2'.
    self assert: class classVariableString = 'ClassVar1 ClassVar2'.
    self assert: class poolDictionaries = 'dictName'.
    self assert: class category = 'Some-Category'.

    self assert: (class includesSelector: #aSelector).
    self assert: (class sourceCodeAt: #aSelector) = 'aSelector
    self shouldImplement'.
    method := class compiledMethodAt: #aSelector.  
    self assert: (method category) = 'a protocol'.

    "Created: / 04-05-2014 / 13:00:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 04-05-2014 / 23:23:58 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_class_and_class_method_generated_as_one_undo_change
    | className class method lastUndoChange |

    lastUndoChange := self lastUndoChange.

    className := 'MockClassForTestingOneUndoChange'.

    self assert: (Smalltalk classNamed: className) isNil.

    builder := CustomSourceCodeBuilder new.
    class := builder createClass.
    class className: className.  

    self assert: (Smalltalk classNamed: className) isNil.

    method := builder createMethod.
    method 
        class: class theMetaclass;
        protocol: 'a protocol';
        source: 'aSelector
        ^ nil'.

    self assert: (Smalltalk classNamed: className) isNil.
    self assert: lastUndoChange = (self lastUndoChange).

    builder compile.

    self assert: (Smalltalk classNamed: className) isNil not.
    self assert: (lastUndoChange = (self lastUndoChange)) not.
    self assert: ((Smalltalk classNamed: className) class includesSelector: #aSelector).
    self assert: ((Smalltalk classNamed: className) includesSelector: #aSelector) not.

    changeManager undoOperation.

    self assert: (Smalltalk classNamed: className) isNil.
    self assert: lastUndoChange = (self lastUndoChange).

    "Created: / 17-04-2014 / 23:54:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 29-04-2014 / 19:45:33 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_class_and_instance_method_generated_as_one_undo_change
    | className class method lastUndoChange |

    lastUndoChange := self lastUndoChange.

    className := 'MockClassForTestingOneUndoChange'.

    self assert: (Smalltalk classNamed: className) isNil.

    builder := CustomSourceCodeBuilder new.
    class := builder createClass.
    class className: className.  

    self assert: (Smalltalk classNamed: className) isNil.

    method := builder createMethod.
    method 
        class: class;
        protocol: 'a protocol';
        source: 'aSelector
        ^ nil'.

    self assert: (Smalltalk classNamed: className) isNil.
    self assert: lastUndoChange = (self lastUndoChange).

    builder compile.

    self assert: (Smalltalk classNamed: className) isNil not.
    self assert: (lastUndoChange = (self lastUndoChange)) not.
    self assert: ((Smalltalk classNamed: className) includesSelector: #aSelector).

    changeManager undoOperation.

    self assert: (Smalltalk classNamed: className) isNil.
    self assert: lastUndoChange = (self lastUndoChange).

    "Created: / 17-04-2014 / 23:54:05 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_comment_in_method_generated
    | method class |

    class := self class.

    method := builder createMethod.
    method 
        class: self class;
        protocol: 'a protocol';
        source: 'aSelector
        "a comment"
        ^ nil'.

    self assert: (class includesSelector: #aSelector) not.

    builder execute.

    self assert: (class includesSelector: #aSelector).
    self assert: ((class sourceCodeAt: #aSelector) includesSubString: '"a comment"').
    self assert: (class sourceCodeAt: #aSelector) = 'aSelector
        "a comment"
        ^ nil'.

    class removeSelector: #aSelector.

    self assert: (class includesSelector: #aSelector) not.

    "Created: / 27-04-2014 / 15:57:17 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 27-04-2014 / 17:17:26 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_one_method_for_multiple_classes
    | classCollection classNames |

    classCollection := OrderedCollection new.
    classNames := Array with: 'DummyClassForTestCase1' with: 'DummyClassForTestCase2'.

    classNames do: [ :className |
        classCollection add: (builder className: className; createClass).
    ].

    builder
        classes: classCollection;
        protocol: 'a protocol';
        source: '`@selector
            self shouldImplement';
        replace: '`@selector' with: 'aSelector';
        createMethod.

    builder execute.

    classNames do: [ :className |
        | class method |

        class := Smalltalk classNamed: className.
        self assert: (class includesSelector: #aSelector).
        self assert: (class sourceCodeAt: #aSelector) = 'aSelector
    self shouldImplement'.
        method := class compiledMethodAt: #aSelector.  
        self assert: (method category) = 'a protocol'.
    ].

    "Created: / 02-05-2014 / 23:31:07 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 04-05-2014 / 11:51:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

