"{ Package: 'jn:refactoring_custom' }"

CustomCodeGenerator subclass:#CustomSubclassResponsibilityCodeGenerator
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!


!CustomSubclassResponsibilityCodeGenerator class methodsFor:'accessing-presentation'!

description
    "Returns more detailed description of the receiver"

    ^ 'Generates method stubs marked by: self subclassResponsibility'

    "Modified: / 08-04-2014 / 21:40:35 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

label
    "Returns show label describing the receiver. This label
     is used in UI as menu item/tree item label."

    ^ 'Subclass responsibility methods'

    "Modified: / 08-04-2014 / 21:38:02 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGenerator class methodsFor:'queries'!

availableInContext: aCustomContext
    "Returns true if the generator/refactoring is available in given
     context, false otherwise. 

     Called by the UI to figure out what generators / refactorings 
     are available at given point. See class CustomContext for details."

    ^ aCustomContext selectedClasses notEmptyOrNil

!

availableInPerspective:aCustomPerspective 
    "Returns true if the generator/refactoring is available in given
     perspective, false otherwise.

     Called by the UI to figure out what generators / refactorings
     to show"

    ^ aCustomPerspective isClassPerspective

! !

!CustomSubclassResponsibilityCodeGenerator methodsFor:'code generation'!

createSubclassResponsibilityForClass: aClass

    | missingRequiredSelectors |

    missingRequiredSelectors := CodeGeneratorTool missingRequiredProtocolFor: aClass.

    missingRequiredSelectors do: [ :selector |
        | method superclassMethod |

        superclassMethod := (self methodForSuperclassSelector: selector class: aClass).

        method := builder createMethod.
        method
            class: aClass;
            protocol: superclassMethod category;
            source: ('`@selector
                "a comment"

                self shouldImplement');
            replace: '`@selector' with: superclassMethod methodDefinitionTemplate asSymbol.

        "/ Other API variant:
        "/
        "/ builder
        "/     class: class;
        "/     protocol: aProtocol;
        "/     source: '`@selector:
        "/       self shouldImplement';
        "/     replace: 'selector' with: selector.
        "/     createMethod.
    ]

    "Created: / 09-04-2014 / 20:16:55 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 27-04-2014 / 17:42:58 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGenerator methodsFor:'executing'!

buildInContext:aCustomContext

    aCustomContext selectedClasses do: [ :class |
        self
            createSubclassResponsibilityForClass: class theNonMetaclass;
            createSubclassResponsibilityForClass: class theMetaclass.
    ]

    "Created: / 08-04-2014 / 21:33:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 13-05-2014 / 22:04:39 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGenerator methodsFor:'queries'!

methodForSuperclassSelector: aSelector class: aClass
    "retrieve method under given selector in class 
    superclass or in superclass superclass until method is found
    or Object is reached"

    | method |

    (aClass == Object) ifTrue: [
        ^ nil
    ].

    method := aClass superclass compiledMethodAt: aSelector asSymbol.
    method isNil ifTrue: [
        ^ self methodForSuperclassSelector: aSelector class: (aClass superclass)
    ] ifFalse: [
        ^ method
    ]

    "Created: / 12-04-2014 / 17:17:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGenerator class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

