"{ Package: 'jn:refactoring_custom' }"

TestCase subclass:#CustomSubclassResponsibilityCodeGeneratorTests
	instanceVariableNames:'codeGenerator mockSuperClass mockClass'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom-Tests'
!

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'initialization & release'!

setUp

    codeGenerator := CustomSubclassResponsibilityCodeGenerator new.
    codeGenerator dialog: CustomDialogMock new.
    mockSuperClass := self createClass: 'MockSuperClassForTestCase' superClassName: 'Object'.
    mockClass := self createClass: 'MockClassForTestCase' superClassName: (mockSuperClass new className).

    "Created: / 14-04-2014 / 17:16:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 13-05-2014 / 22:36:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

tearDown

    mockSuperClass removeFromSystem.
    mockClass removeFromSystem.

    "Created: / 14-04-2014 / 17:16:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'private'!

createClass: aClassName superClassName: aSuperClassName
    | class |

    (Compiler evaluate:  
        aSuperClassName, ' subclass:#', aClassName, '
            instanceVariableNames:''''
            classVariableNames:''''
            poolDictionaries:''''
            category:''''' asString).

    class := Smalltalk classNamed: aClassName.
    ^ class.

    "Created: / 14-04-2014 / 17:26:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-04-2014 / 20:54:13 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

createMethod: aClass protocol: aProtocol source: aSource
    | builder method |

    builder := CustomSourceCodeBuilder new.
    method := builder createMethod.
    method
        class: aClass;
        protocol: aProtocol;
        source: aSource;
        compile.

    "Created: / 14-04-2014 / 18:26:35 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomSubclassResponsibilityCodeGeneratorTests methodsFor:'tests'!

test_method_from_superclass_retrieved
    | method |

    self assert: (codeGenerator methodForSuperclassSelector: 'initialize' class: Object) isNil.

    "/ Instance method
    self createMethod: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    self shouldImplement'.

    method := codeGenerator methodForSuperclassSelector: #instanceMethod: class: mockClass.
    self assert: 'instance-protocol' = method category.
    Transcript showCR: 'TMP: ', method methodDefinitionTemplate.
    self assert: 'instanceMethod:aParam' = method methodDefinitionTemplate.

    "/ Class method
    self createMethod: mockSuperClass class protocol: 'class-protocol' source: 'classMethod: aParam
    self shouldImplement'.

    method := codeGenerator methodForSuperclassSelector: 'classMethod:' class: mockClass class.
    self assert: 'class-protocol' = method category.
    self assert: 'classMethod:aParam' = method methodDefinitionTemplate.

    "Created: / 14-04-2014 / 18:12:57 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 15-04-2014 / 21:10:56 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_none_subclass_responsibility
    | context |

    context := CustomSubContext new.
    context addSelectedClass: mockClass.

    self createMethod: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    ^ self'.

    self assert: (mockClass includesSelector: #instanceMethod:) not. 
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    codeGenerator executeInContext: context.

    self assert: (mockClass includesSelector: #instanceMethod:) not.
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    "Created: / 13-05-2014 / 21:49:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_subclass_responsibility_class_method
    | context |

    context := CustomSubContext new.
    context addSelectedClass: mockClass.

    self createMethod: mockSuperClass class protocol: 'class-protocol' source: 'classMethod: aParam
    self subclassResponsibility'.

    self assert: (mockClass includesSelector: #classMethod:) not.
    self assert: (mockClass class includesSelector: #classMethod:) not.

    codeGenerator executeInContext: context.

    self assert: (mockClass includesSelector: #classMethod:) not.
    self assert: (mockClass class includesSelector: #classMethod:). 

    self assert: (mockClass class sourceCodeAt: #classMethod:) = ('classMethod:aParam
    self shouldImplement').

    "Created: / 15-04-2014 / 22:05:19 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 26-04-2014 / 22:22:59 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

test_subclass_responsibility_instance_method
    | context |

    context := CustomSubContext new.
    context addSelectedClass: mockClass.

    self createMethod: mockSuperClass protocol: 'instance-protocol' source: 'instanceMethod: aParam
    self subclassResponsibility'.

    self assert: (mockClass includesSelector: #instanceMethod:) not. 
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    codeGenerator executeInContext: context.

    self assert: (mockClass includesSelector: #instanceMethod:).
    self assert: (mockClass class includesSelector: #instanceMethod:) not.

    self assert: (mockClass sourceCodeAt: #instanceMethod:) = ('instanceMethod:aParam
    self shouldImplement').

    "Created: / 14-04-2014 / 18:10:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 26-04-2014 / 22:23:28 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

